@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -94,6 +94,10 @@ static void         thunar_details_view_row_activated           (GtkTreeView
                                                                  GtkTreePath            *path,
                                                                  GtkTreeViewColumn      *column,
                                                                  ThunarDetailsView      *details_view);
+static void         thunar_details_view_row_changed             (GtkTreeView            *tree_view,
+                                                                 GtkTreePath            *path,
+                                                                 GtkTreeViewColumn      *column,
+                                                                 ThunarDetailsView      *details_view);
 static void         thunar_details_view_columns_changed         (ThunarColumnModel      *column_model,
                                                                  ThunarDetailsView      *details_view);
 static void         thunar_details_view_zoom_level_changed      (ThunarDetailsView      *details_view);
@@ -229,6 +233,8 @@ thunar_details_view_init (ThunarDetailsView *details_view)
   /* connect to the default column model */
   details_view->column_model = thunar_column_model_get_default ();
   g_signal_connect (G_OBJECT (details_view->column_model), "columns-changed", G_CALLBACK (thunar_details_view_columns_changed), details_view);
+  g_signal_connect_after (G_OBJECT (THUNAR_STANDARD_VIEW (details_view)->model), "row-changed",
+                          G_CALLBACK (thunar_details_view_row_changed), details_view);
 
   /* allocate the shared right-aligned text renderer */
   right_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 1.0f, NULL);
@@ -653,8 +659,10 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
                                         ThunarDetailsView *details_view)
 {
   GtkTreeSelection  *selection;
-  GtkTreePath       *path;
+  GtkTreePath       *path = NULL;
   GtkTreeIter        iter;
+  GtkTreeViewColumn *column;
+  GtkTreeViewColumn *first_column;
   ThunarFile        *file;
   GtkAction         *action;
   ThunarPreferences *preferences;
@@ -665,87 +673,121 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
   if (G_UNLIKELY (event->window != gtk_tree_view_get_bin_window (tree_view)))
     return FALSE;
 
-  /* we unselect all selected items if the user clicks on an empty
-   * area of the treeview and no modifier key is active.
-   */
+  /* get the current selection */
+  selection = gtk_tree_view_get_selection (tree_view);
+
+  /* get the first column of the tree view */
+  first_column = gtk_tree_view_get_column (tree_view, 0);
+
+  /* unselect all selected items if the user clicks on an empty area
+   * of the treeview and no modifier key is active */
   if ((event->state & gtk_accelerator_get_default_mod_mask ()) == 0
-      && !gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, NULL, NULL, NULL, NULL))
-    {
-      selection = gtk_tree_view_get_selection (tree_view);
+      && !gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, &column, NULL, NULL))
       gtk_tree_selection_unselect_all (selection);
+
+  /* if the user clicked on a row with the left button */
+  if (path != NULL && event->type == GDK_BUTTON_PRESS && event->button == 1)
+    {
+      GtkTreePath       *cursor_path;
+
+      /* grab the tree view */
+      gtk_widget_grab_focus (GTK_WIDGET (tree_view));
+
+      gtk_tree_view_get_cursor (tree_view, &cursor_path, NULL);
+      if (cursor_path != NULL)
+        {
+          gtk_tree_path_free (cursor_path);
+
+          if (column != first_column)
+            gtk_tree_selection_unselect_all (selection);
+
+          /* do not start rubber banding from the first column */
+          if (!gtk_tree_selection_path_is_selected (selection, path)
+              && column == first_column)
+            {
+              /* the clicked row does not have the focus and is not
+               * selected, so make it the new selection (start) */
+              gtk_tree_selection_unselect_all (selection);
+              gtk_tree_selection_select_path (selection, path);
+
+              gtk_tree_path_free (path);
+
+              /* return FALSE to not abort dragging */
+              return FALSE;
+            }
+          gtk_tree_path_free (path);
+        }
     }
 
   /* open the context menu on right clicks */
   if (event->type == GDK_BUTTON_PRESS && event->button == 3)
     {
-      selection = gtk_tree_view_get_selection (tree_view);
-      if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL))
+      if (path == NULL)
         {
-          /* select the path on which the user clicked if not selected yet */
-          if (!gtk_tree_selection_path_is_selected (selection, path))
+          /* open the context menu */
+          thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
+        }
+      else
+        {
+          if (column != first_column)
             {
-              /* only select an item if Control is not pressed, else we
-               * use this to popup the current directory actionW */
-              gtk_tree_selection_unselect_all (selection);
-              if ((event->state & GDK_CONTROL_MASK) == 0)
-                gtk_tree_selection_select_path (selection, path);
+              /* if the clicked path is not selected, unselect all other paths */
+              if (!gtk_tree_selection_path_is_selected (selection, path))
+                gtk_tree_selection_unselect_all (selection);
+
+              /* queue the menu popup */
+              thunar_standard_view_queue_popup (THUNAR_STANDARD_VIEW (details_view), event);
+            }
+          else
+            {
+              /* select the clicked path if necessary */
+              if (!gtk_tree_selection_path_is_selected (selection, path))
+                {
+                  gtk_tree_selection_unselect_all (selection);
+                  gtk_tree_selection_select_path (selection, path);
+                }
+
+              /* show the context menu */
+              thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
             }
           gtk_tree_path_free (path);
         }
 
-      /* open the context menu */
-      thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), event->button, event->time);
-
       return TRUE;
     }
-  else if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS) && event->button == 2)
+  else if (event->type == GDK_BUTTON_PRESS && event->button == 2)
     {
       /* determine the path to the item that was middle-clicked */
       if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL))
         {
           /* select only the path to the item on which the user clicked */
-          selection = gtk_tree_view_get_selection (tree_view);
           gtk_tree_selection_unselect_all (selection);
           gtk_tree_selection_select_path (selection, path);
 
-          /* if the event was a double-click or we are in single-click mode, then
-           * we'll open the file or folder (folder's are opened in new windows)
-           */
-          if (G_LIKELY (event->type == GDK_2BUTTON_PRESS || exo_tree_view_get_single_click (EXO_TREE_VIEW (tree_view))))
+          /* determine the file for the path */
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (details_view)->model), &iter, path);
+          file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (details_view)->model, &iter);
+          if (G_LIKELY (file != NULL) && thunar_file_is_directory (file))
             {
-              /* determine the file for the path */
-              gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (details_view)->model), &iter, path);
-              file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (details_view)->model, &iter);
-              if (G_LIKELY (file != NULL))
-                {
-                  /* determine the action to perform depending on the type of the file */
-                  if (thunar_file_is_directory (file))
-                    {
-                      /* lookup setting if we should open in a tab or a window */
-                      preferences = thunar_preferences_get ();
-                      g_object_get (preferences, "misc-middle-click-in-tab", &in_tab, NULL);
-                      g_object_unref (preferences);
+              /* lookup setting if we should open in a tab or a window */
+              preferences = thunar_preferences_get ();
+              g_object_get (preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+              g_object_unref (preferences);
 
-                      /* holding ctrl inverts the action */
-                      if ((event->state & GDK_CONTROL_MASK) != 0)
-                        in_tab = !in_tab;
+              /* holding ctrl inverts the action */
+              if ((event->state & GDK_CONTROL_MASK) != 0)
+                  in_tab = !in_tab;
 
-                      action_name = in_tab ? "open-in-new-tab" : "open-in-new-window";
-                    }
-                  else
-                    {
-                      action_name = "open";
-                    }
+              action_name = in_tab ? "open-in-new-tab" : "open-in-new-window";
 
-                  action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, action_name);
-      
-                  /* emit the action */
-                  if (G_LIKELY (action != NULL))
-                    gtk_action_activate (action);
+              action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, action_name);
 
-                  /* release the file reference */
-                  g_object_unref (G_OBJECT (file));
-                }
+              /* emit the action */
+              if (G_LIKELY (action != NULL))
+                  gtk_action_activate (action);
+
+              /* release the file reference */
+              g_object_unref (G_OBJECT (file));
             }
 
           /* cleanup */
@@ -801,6 +843,19 @@ thunar_details_view_row_activated (GtkTreeView       *tree_view,
 
 
 
+static void
+thunar_details_view_row_changed (GtkTreeView       *tree_view,
+                                 GtkTreePath       *path,
+                                 GtkTreeViewColumn *column,
+                                 ThunarDetailsView *details_view)
+{
+  _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (details_view));
+
+  gtk_widget_queue_draw (GTK_WIDGET (details_view));
+}
+
+
+
 static void
 thunar_details_view_columns_changed (ThunarColumnModel *column_model,
                                      ThunarDetailsView *details_view)
