@@ -387,7 +387,7 @@ thunar_path_entry_button_press_event (GtkWidget      *widget,
       /* consume the event */
       path_entry->drag_button = event->button;
       path_entry->drag_x = event->x;
-      path_entry->drag_x = event->y;
+      path_entry->drag_y = event->y;
       return TRUE;
     }
 
@@ -538,6 +538,7 @@ thunar_path_entry_changed (GtkEditable *editable)
   ThunarFolder       *folder;
   GtkTreeModel       *model;
   const gchar        *text;
+  gchar              *escaped_text;
   ThunarFile         *current_folder;
   ThunarFile         *current_file;
   GFile              *folder_path = NULL;
@@ -558,7 +559,9 @@ thunar_path_entry_changed (GtkEditable *editable)
   if (G_UNLIKELY (exo_str_looks_like_an_uri (text)))
     {
       /* try to parse the URI text */
-      file_path = g_file_new_for_uri (text);
+      escaped_text = g_uri_escape_string (text, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, TRUE);
+      file_path = g_file_new_for_uri (escaped_text);
+      g_free (escaped_text);
 
       /* use the same file if the text assumes we're in a directory */
       if (g_str_has_suffix (text, "/"))
@@ -915,10 +918,16 @@ thunar_path_entry_match_func (GtkEntryCompletion *completion,
       /* determine the real file name for the iter */
       gtk_tree_model_get (model, iter, THUNAR_COLUMN_FILE_NAME, &name, -1);
       name_normalized = g_utf8_normalize (name, -1, G_NORMALIZE_ALL);
-      g_free (name);
+      if (G_LIKELY (name_normalized != NULL))
+        g_free (name);
+      else
+        name_normalized = name;
 
       /* check if we have a match here */
-      matched = g_str_has_prefix (name_normalized, last_slash);
+      if (name_normalized != NULL)
+        matched = g_str_has_prefix (name_normalized, last_slash);
+      else
+        matched = FALSE;
 
       /* cleanup */
       g_free (name_normalized);
@@ -1076,7 +1085,7 @@ thunar_path_entry_check_completion_idle (gpointer user_data)
 
   GDK_THREADS_ENTER ();
 
-  /* check if the user entered atleast part of a filename */
+  /* check if the user entered at least part of a filename */
   text = gtk_entry_get_text (GTK_ENTRY (path_entry));
   if (*text != '\0' && text[strlen (text) - 1] != '/')
     {
@@ -1145,9 +1154,11 @@ void
 thunar_path_entry_set_current_file (ThunarPathEntry *path_entry,
                                     ThunarFile      *current_file)
 {
-  GFile *file;
-  gchar *text;
-  gchar *unescaped;
+  GFile    *file;
+  gchar    *text;
+  gchar    *unescaped;
+  gchar    *tmp;
+  gboolean  is_uri = FALSE;
 
   _thunar_return_if_fail (THUNAR_IS_PATH_ENTRY (path_entry));
   _thunar_return_if_fail (current_file == NULL || THUNAR_IS_FILE (current_file));
@@ -1168,16 +1179,33 @@ thunar_path_entry_set_current_file (ThunarPathEntry *path_entry,
 
           /* if there is no local path, use the URI (which always works) */
           if (text == NULL)
-            text = g_file_get_uri (file);
+            {
+              text = g_file_get_uri (file);
+              is_uri = TRUE;
+            }
         }
       else
         {
           /* not a native file, use the URI */
           text = g_file_get_uri (file);
+          is_uri = TRUE;
+        }
+
+      /* if the file is a directory, end with a / to avoid loading the parent
+       * directory which is probably not something the user wants */
+      if (thunar_file_is_directory (current_file)
+          && !g_str_has_suffix (text, "/"))
+        {
+          tmp = g_strconcat (text, "/", NULL);
+          g_free (text);
+          text = tmp;
         }
     }
 
-  unescaped = g_uri_unescape_string (text, NULL);
+  if (is_uri)
+    unescaped = g_uri_unescape_string (text, NULL);
+  else
+    unescaped = g_strdup (text);
   g_free (text);
 
   /* setup the entry text */
